"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const clone_1 = tslib_1.__importDefault(require("./utils/clone"));
const get_1 = tslib_1.__importDefault(require("./utils/get"));
function numerical(v) {
    if (v instanceof Date)
        return v.getTime();
    if (Array.isArray(v))
        return v.length;
    if (v instanceof Map || v instanceof Set)
        return v.size;
    return Number(v);
}
const sort = (array, opts) => {
    if (typeof opts === "function")
        return array.sort(opts);
    const target = (opts === null || opts === void 0 ? void 0 : opts.copy) ? (opts.copy === "deep" ? clone_1.default(array) : [...array]) : array;
    target.sort(!(opts === null || opts === void 0 ? void 0 : opts.by)
        ? undefined
        : (a, b) => {
            //@ts-ignore
            const A = get_1.default(a, opts.by);
            //@ts-ignore
            const B = get_1.default(b, opts.by);
            if (typeof A === "undefined" && typeof B === "undefined")
                return 0;
            if (typeof A === "undefined")
                return -1;
            if (typeof B === "undefined")
                return 1;
            if (opts.type === "string.length")
                return A.length - B.length;
            if (opts.type === "alphabetically")
                return A.localeCompare(B);
            if (opts.type === "number")
                return numerical(A) - numerical(B);
            if (opts.type === "boolean")
                return A && B ? 0 : A ? 1 : B ? -1 : 0;
            if (A instanceof Date && B instanceof Date)
                return A.getTime() - B.getTime();
            if (Array.isArray(A) && Array.isArray(B))
                return A.length - B.length;
            if ((A instanceof Map && B instanceof Map) || (A instanceof Set && B instanceof Set))
                return A.size - B.size;
            if (typeof A === "string" && typeof B === "string")
                return A.localeCompare(B);
            if (typeof A === "boolean" && typeof B === "boolean")
                return A && B ? 0 : A ? 1 : B ? -1 : 0;
            return numerical(A) - numerical(B);
        });
    const desc = (opts === null || opts === void 0 ? void 0 : opts.order) === -1 || (typeof (opts === null || opts === void 0 ? void 0 : opts.order) === "string" && opts.order.toLowerCase().startsWith("desc"));
    if (desc)
        target.reverse();
    return target;
};
exports.default = sort;
